from pathlib import Path
from tinydb import TinyDB, Query

from yoku.consts import KEY_ITEM_ID, KEY_CHAT_ID
from yoku.scrape import search
from yoku.botutils import notify

DB_PATH = Path(__file__).resolve().parent / "db.json"

TABLE_QUERIES = "queries"
KEY_QUERY = "query"


db = TinyDB(str(DB_PATH))


def add_query(chat_id, query):
    """
    Add a user query to the database
    """

    queries_table = db.table(TABLE_QUERIES)
    Item = Query()

    # Skip if already exists
    if queries_table.search((Item[KEY_QUERY] == query) & (Item[KEY_CHAT_ID] == chat_id)):
        return

    queries_table.insert({KEY_QUERY: query, KEY_CHAT_ID: chat_id})


def remove_query(chat_id, query):
    """
    Remove a user query from the database
    """

    queries_table = db.table(TABLE_QUERIES)
    Item = Query()

    queries_table.remove((Item[KEY_QUERY] == query)
                         & (Item[KEY_CHAT_ID] == chat_id))


def list_queries(chat_id):
    queries_table = db.table(TABLE_QUERIES)
    Item = Query()

    queries = queries_table.search(Item[KEY_CHAT_ID] == chat_id)

    return [x[KEY_QUERY] for x in queries]


def search_and_notify(bot):
    """
    Iterate through saved queries and perform search
    Cache results and notify users of new results
    """

    queries_table = db.table(TABLE_QUERIES)

    for row in queries_table:
        chat_id = row[KEY_CHAT_ID]
        query = row[KEY_QUERY]

        results = search(query)
        cache_table = db.table(chat_id)

        for result in results:
            item_id = result[KEY_ITEM_ID]

            Item = Query()
            if not cache_table.search(Item[KEY_ITEM_ID] == item_id):
                notify(chat_id, result, bot)

            cache_table.upsert(result, Item[KEY_ITEM_ID] == item_id)



import re
from urllib import parse
from bs4 import BeautifulSoup
import requests

from yoku.consts import KEY_TITLE, KEY_BUYNOW_PRICE, KEY_CURRENT_PRICE, KEY_END_TIMESTAMP, KEY_IMAGE, KEY_ITEM_ID, KEY_POST_TIMESTAMP, KEY_START_PRICE, KEY_START_TIMESTAMP, KEY_URL

# Always search sorted by new in descending order
YAHUOKU_SEARCH_TEMPLATE = r"https://auctions.yahoo.co.jp/search/search?p={query}&b={start}&n={count}&s1=new&o1=d"

POST_TIMESTAMP_REGEX = r"^.*i-img\d+x\d+-(\d{10}).*$"
AUCTION_TIMESTAMP_REGEX = r"^.*etm=(\d{10}),stm=(\d{10}).*$"


def get_raw_results(query: str):
    """
    Return raw html page content of the results from the search query
    """

    # start cannot be more than 15000
    url = YAHUOKU_SEARCH_TEMPLATE.format(query=parse.quote_plus(query), start=1, count=100)
    print(f"[GET] {url}")

    r = requests.get(url)

    return r.text


def parse_raw_results(raw: str):
    """
    Parse a raw html page of search results and return a list of result dicts
    """

    results = []
    soup = BeautifulSoup(raw, "lxml")

    product_details = soup.find_all("div", class_="Product__detail")
    for product_detail in product_details:
        product_bonuses = product_detail.find_all(
            "div", class_="Product__bonus")
        product_titlelinks = product_detail.find_all(
            "a", class_="Product__titleLink")

        if not product_bonuses or not product_titlelinks:
            continue

        product_bonus = product_bonuses[0]
        product_titlelink = product_titlelinks[0]

        auction_title = product_titlelink["data-auction-title"]
        auction_img = product_titlelink["data-auction-img"]
        href = product_titlelink["href"]
        cl_params = product_titlelink["data-cl-params"]

        match = re.match(POST_TIMESTAMP_REGEX, auction_img)
        if not match:
            continue

        post_timestamp = int(match.group(1))

        match = re.match(AUCTION_TIMESTAMP_REGEX, cl_params)
        if not match:
            continue

        end_timestamp = int(match.group(1))
        start_timestamp = int(match.group(2))

        auction_id = product_bonus["data-auction-id"]
        auction_buynowprice = product_bonus["data-auction-buynowprice"]
        auction_price = product_bonus["data-auction-price"]
        auction_startprice = product_bonus["data-auction-startprice"]

        result = {
            KEY_TITLE: auction_title,
            KEY_IMAGE: auction_img,
            KEY_URL: href,

            KEY_POST_TIMESTAMP: post_timestamp,
            KEY_END_TIMESTAMP: end_timestamp,
            KEY_START_TIMESTAMP: start_timestamp,

            KEY_ITEM_ID: auction_id,
            KEY_BUYNOW_PRICE: auction_buynowprice,
            KEY_CURRENT_PRICE: auction_price,
            KEY_START_PRICE: auction_startprice,
        }

        results.append(result)

    return results


def search(query: str):
    """
    Search for query and return a list of result dicts
    """

    raw = get_raw_results(query)

    return parse_raw_results(raw)



from datetime import datetime

from yoku.consts import KEY_TITLE, KEY_POST_TIMESTAMP, KEY_START_PRICE, KEY_URL


def send_message(chat_id, msg, bot):
    """
    Send a message without throwing
    """

    try:
        bot.sendMessage(chat_id, msg)
    except Exception as e:
        print(f"[TELEGRAM EXCEPTION] {e}")


def notify(chat_id, result, bot):
    """
    Notify a user of a new result
    """

    msg = f"{result[KEY_URL]}\n{result[KEY_TITLE]}\n{datetime.fromtimestamp(result[KEY_POST_TIMESTAMP]).strftime('%d/%m/%Y %I:%M:%S %p')}\n{result[KEY_START_PRICE]}¥"
    send_message(chat_id, msg, bot)



import os
import time
from random import randint
from threading import Thread

import telegram
from telegram.ext import Updater, CommandHandler

from yoku.consts import BOT_TOKEN_ENVVAR
from yoku.database import add_query, remove_query, search_and_notify, list_queries
from yoku.botutils import send_message

from pathlib import Path


# Retrieve environment variables
BOT_TOKEN = os.environ.get(BOT_TOKEN_ENVVAR)
if not BOT_TOKEN:
    print(f"`{BOT_TOKEN_ENVVAR}` environment variable not set")
    exit(1)

bot = telegram.Bot(BOT_TOKEN)


def get_message_data(update):
    """
    Parse Telegram `update` object and return chat ID and query text
    """

    message_json = update.message
    chat_id = str(message_json["chat"]["id"])
    message_full = message_json["text"]
    query_text = " ".join(message_full.split()[1:])

    return chat_id, query_text


def add(update, context):
    """
    Add a query for user
    """

    chat_id, query_text = get_message_data(update)
    if not query_text:
        return

    add_query(chat_id, query_text)


def rm(update, context):
    """
    Remove a query for user
    """

    chat_id, query_text = get_message_data(update)
    if not query_text:
        return

    remove_query(chat_id, query_text)


def ls(update, context):
    """
    List queries of a user
    """

    chat_id, _ = get_message_data(update)
    queries = list_queries(chat_id)
    if not queries:
        return

    send_message(chat_id, "\n".join(queries), bot)


def force(update, context):
    """
    Force fetch queries 
    """

    search_and_notify(bot)


def main_loop():
    while True:
        search_and_notify(bot)
        s = randint(1000, 10000)
        print(f"Sleeping for {s}s")
        time.sleep(s)


def main():
    # Initialize Telegram bot
    updater = Updater(BOT_TOKEN)

    # Add command handlers
    updater.dispatcher.add_handler(CommandHandler("add", add))
    updater.dispatcher.add_handler(CommandHandler("rm", rm))
    updater.dispatcher.add_handler(CommandHandler("ls", ls))
    updater.dispatcher.add_handler(CommandHandler("force", force))

    # Start the api polling loop in a thread
    Thread(target=main_loop).start()

    # Start the Telegram bot
    updater.start_polling()

    print(f"Bot started")
    updater.idle()


if __name__ == "__main__":
    main()



BOT_TOKEN_ENVVAR = "YOKU_BOT_TOKEN"

KEY_TITLE = "title"
KEY_URL = "url"
KEY_IMAGE = "img"
KEY_POST_TIMESTAMP = "post_ts"
KEY_END_TIMESTAMP = "end_ts"
KEY_START_TIMESTAMP = "start_ts"
KEY_BUYNOW_PRICE = "buynow_price"
KEY_CURRENT_PRICE = "curr_price"
KEY_START_PRICE = "start_price"
KEY_ITEM_ID = "item_id"
KEY_CHAT_ID = "chat_id"



from setuptools import setup


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="Yoku",
    version="1.0.3",
    packages=["yoku"],
    description="A minimal Yahoo! Auctions scraper.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[
        "requests>=2.26.0",
        "python-telegram-bot>=13.15",
        "beautifulsoup4>=4.11.1",
        "tinydb>=4.7.0",
        "lxml>=4.9.1"
    ],
    entry_points={
        "console_scripts": [
            "yoku = yoku.bot:main",
        ]
    },
)
